#include "barrier.h"
#include <iostream>

Barrier::Barrier(int count)
	: count(count)
{
	pthread_mutex_init(&mutex, NULL);
	pthread_cond_init(&cond, NULL);
	state = count;
}

Barrier::~Barrier()
{
	pthread_mutex_destroy(&mutex);
	pthread_cond_destroy(&cond);
	if (state != count)
	{
		std::cout << "FATAL ERROR: Barrier object destroyed while a thread is waiting!" << std::endl;
	}
}

void
Barrier::wait()
{
	int curr_cycle, cancel;

	pthread_mutex_lock(&mutex);
	curr_cycle = cycle;
	
	if (--state == 0)
	{
		state = count;
		cycle++;
		pthread_cond_broadcast(&cond);
	}
	else
	{
		pthread_setcancelstate (PTHREAD_CANCEL_DISABLE, &cancel);
		
		while (cycle == curr_cycle)
		{
			pthread_cond_wait(&cond, &mutex);
		}
	}

	pthread_mutex_unlock(&mutex);
}
