#include "camera.h"

#define RAD(deg) ((deg) * M_PI / 180.0)

#include <iostream>

Camera::Camera()
{
	// world coordinates
	xrot = -80.0;
	zrot = 45.0;
	x = -5.0;
	y = -5.0;
	z = 5.0;
	speed = 10.0;
	rotspeed = 10.0;
	forward = backward = left = right = up = down = rotleft = rotright = false;
	timer.increment();
}

void 
Camera::updatePosition()
{
	timer.increment();
	double time = timer.ddelta();
	double dist = speed * time;
	double rotdist = rotspeed * time;
	
	if (forward) 
	{
		y -= dist * cos(RAD(zrot)) * sin(RAD(xrot));
		x -= dist * sin(RAD(zrot)) * sin(RAD(xrot));
		z -= dist * cos(RAD(xrot));
	} 
	else if (backward)
	{
		y += dist * cos(RAD(zrot)) * sin(RAD(xrot));
		x += dist * sin(RAD(zrot)) * sin(RAD(xrot));
		z += dist * cos(RAD(xrot));
	}
	
	if (right)
	{
		y -= dist * sin(RAD(zrot));
		x += dist * cos(RAD(zrot));
	}
	else if (left)
	{
		y += dist * sin(RAD(zrot));
		x -= dist * cos(RAD(zrot));
	}
	if (rotleft)
	{
		zrot -= rotdist * rotspeed;	
	}
	else if (rotright)
	{
		zrot += rotdist * rotspeed;	
	}
	if (up)
	{
		xrot -= rotdist * rotspeed;	
	}
	else if (down)
	{
		xrot += rotdist * rotspeed;
	}
}

void 
Camera::setPerspective()
{
	int viewport[4];
	glGetIntegerv(GL_VIEWPORT, viewport);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(35.0f, (GLfloat) viewport[2] / (GLfloat) viewport[3], 2.0f, 80.0f);

	glMatrixMode(GL_MODELVIEW);	
	glLoadIdentity();
	glRotatef(xrot, 1.0, 0.0, 0.0);
	glRotatef(zrot, 0.0, 0.0, 1.0);
	glTranslatef(-x, -y, -z);

	setLighting();
}

void
Camera::setLighting()
{
	glEnable(GL_LIGHTING);
	//glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);
	glEnable(GL_LIGHT2);
	glEnable(GL_LIGHT3);
	glEnable(GL_LIGHT4);

	GLfloat global_ambient[] = { 1.0f, 1.0f, 1.0, 1.0f };
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, global_ambient);
	
	// Create light components
	GLfloat diffuseLight[] = { 0.2f, 0.2f, 0.2, 1.0f };
	GLfloat specularLight[] = { 0.1f, 0.1f, 0.1f, 1.0f };
	//GLfloat position1[] = { -10.0f, -10.0f, 0.0f, 1.0f };
	GLfloat position2[] = { -10.0f, 60.0f, 0.0f, 1.0f };
	GLfloat position3[] = { 60.0f, -10.0f, 0.0f, 1.0f };
	GLfloat position4[] = { 60.0f, 60.0f, 0.0f, 1.0f };

	GLfloat position5[] = { -20.0f, -20.0f, 30.0f, 1.0f };
	GLfloat diffuseLight2[] = { .7f, .7f, .7, 1.0f };
	GLfloat specularLight2[] = { .6f, .6f, .6, 1.0f };

	//glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
	//glLightfv(GL_LIGHT0, GL_POSITION, position1);
	
	diffuseLight[0] = diffuseLight[1] = diffuseLight[2] = 0.1f;
	glLightfv(GL_LIGHT1, GL_DIFFUSE, diffuseLight);
	glLightfv(GL_LIGHT1, GL_POSITION, position2);

	glLightfv(GL_LIGHT2, GL_DIFFUSE, diffuseLight);
	glLightfv(GL_LIGHT2, GL_POSITION, position3);

	diffuseLight[0] = diffuseLight[1] = diffuseLight[2] = 0.5f;
	glLightfv(GL_LIGHT3, GL_DIFFUSE, diffuseLight);
	glLightfv(GL_LIGHT3, GL_POSITION, position4);

	glLightfv(GL_LIGHT4, GL_DIFFUSE, diffuseLight2);
	glLightfv(GL_LIGHT4, GL_POSITION, position5);
	//glLightfv(GL_LIGHT4, GL_SPECULAR, specularLight2);
}
	
void
Camera::enable2D()
{
	int viewport[4];
	
	glGetIntegerv(GL_VIEWPORT, viewport);
	
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(0, viewport[2], 0, viewport[3], -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
}

void
Camera::disable2D()
{
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}


