#include "configuration.h"

#include "vector3D.h"

State::State()
{
	paused = false; 
	interrupted = false;
	boundaryVelocity = Vector3D(0.0, 0.0, 0.0);
	drawWindField = false; 
	drawPressureField = false;
	drawWindFieldOutline = false;
	resetWindField = false;
	simulateWind = true; 
	applyWindForce = true; 
	fogEnabled = true;
	render = true;
}

Configuration::Configuration()
{
	snowInvDepth = 0.02;
	printFpsInterval = 5000000;
	sorRelax = 1.7;
	parallelMode = PARALLEL_PIPELINE;
}

Configuration::~Configuration()
{
}

void skipline(std::istream& s)
{
	char buf[512];
	int len = 512;
	s.get();
	s.getline(buf, len);
}

int
Configuration::read(std::istream& in)
{
	skipline(in);
	in >> screenWidth >> screenHeight;
	skipline(in);
	in >> nBoxes;
	nBoxes++; // account for ground
	boxes = new Box[nBoxes];
	
	for (int i = 0; i < nBoxes-1; i++)
	{
		skipline(in);
	   	in >> boxes[i].pos.x >> boxes[i].pos.y >> boxes[i].pos.z;
		in >> boxes[i].size.x >> boxes[i].size.y >> boxes[i].size.z;
		in >> boxes[i].color.x >> boxes[i].color.y >> boxes[i].color.z;
		int res_x, res_y;
		in >> res_x >> res_y;
		boxes[i].res_x = res_x;
		boxes[i].res_y = res_y;
	}

	skipline(in);
	in >> threads;
	skipline(in);
	in >> parallelMode;
	skipline(in);
	in >> maxTriangleSize;
	skipline(in);
	in >> numSnowflakes;
	skipline(in);
	in >> xSize >> ySize >> zSize;
	boxes[nBoxes-1].pos = Vector3D(0.0, 0.0, -0.01);
	boxes[nBoxes-1].size = Vector3D(xSize, ySize, 0.01);
	boxes[nBoxes-1].color = Vector3D(0.199, 0.113, 0.051);
	skipline(in);
	in >> boxes[nBoxes-1].res_x >> boxes[nBoxes-1].res_y;

	skipline(in);
	in >> temperature;
	skipline(in);
	in >> wfPos.x >> wfPos.y >> wfPos.z;
	skipline(in);
	in >> wfSize.x >> wfSize.y >> wfSize.z;
	skipline(in);
	in >> Nx >> Ny >> Nz;
	skipline(in);
	in >> snowflakeVolumeMultiplier;
	skipline(in);
	in >> sorIter;
	skipline(in);
	in >> state.boundaryVelocity.x >> state.boundaryVelocity.y >> state.boundaryVelocity.z;
	skipline(in);
	in >> vorticityEpsilon;
	
	return 0;
}

void
Configuration::printConfiguration()
{
	cout << "Configuration:" << endl;
	cout << "Resolution:                        " << screenWidth << "x" << screenHeight << endl;
	cout << "Number of threads:                 " << threads << endl;
	cout << "Parallelism:                       ";
	if (parallelMode == PARALLEL_SERIAL) cout << "Serial" << endl;
	else if (parallelMode == PARALLEL_DATA) cout << "Data-parallel" << endl;
	else cout << "Pipelined" << endl;
	cout << "Number of snowflakes:              " << numSnowflakes << endl;
	cout << "Simulation domain:                 " << xSize << " " << ySize << " " << zSize << endl;
	cout << "Windfield grid resolution:         " << Nx << "x" << Ny << "x" << Nz << endl;
	cout << "SOR iterations:                    " << sorIter << endl;
	cout << "SOR relaxation parameter:          " << sorRelax << endl;
	cout << "Number of boxes (inc. ground):     " << nBoxes << endl;
	cout << "Temperature:                       " << temperature << endl;
	cout << "Snowflake volume multiplier        " << snowflakeVolumeMultiplier << endl;
}
