#ifndef CONFIGURATION_H_
#define CONFIGURATION_H_

#define PARALLEL_SERIAL		(1)
#define PARALLEL_DATA		(2)
#define PARALLEL_PIPELINE   (4)

#include <iostream>
#include "opengl.h"
using namespace std;

#include "box.h"

class State
{
	public:
		State();

		bool paused;
		bool interrupted;
		Vector3D boundaryVelocity;
		bool drawWindField;
		bool drawPressureField;
		bool drawWindFieldOutline;
		bool resetWindField;
		bool simulateWind;
		bool applyWindForce;
		bool fogEnabled;
		SDL_Surface *framebuffer;
		bool render;
};

class Configuration
{
	public:
		Configuration();
		~Configuration();

		int read(std::istream& in);
		void printConfiguration();

		State state;
		
		// CONFIGURATION
		int threads;             /* Number of threads to use*/
		int parallelMode;        /* Serial, data-parallel or pipelined */
		int screenWidth;         /* Size of window */
		int screenHeight;
		
		// SCENE
		int numSnowflakes;       /* Number of snowflakes to simulate */
		float xSize, ySize, zSize; /* Size of simulation domain */
		Vector3D wfPos;          /* wind field position */
		Vector3D wfSize;         /* wind field size */
		int Nx, Ny, Nz;          /* wind field resolution */
		int nBoxes;
		Box *boxes;
		double maxTriangleSize;
		double temperature;
		double snowflakeVolumeMultiplier;
		int sorIter;
		float sorRelax;
		double vorticityEpsilon;
		float snowInvDepth;

		unsigned int printFpsInterval;
};

#endif /* CONFIGURATION_H_ */

