#include "falling_snow_simulation.h"

FallingSnowSimulation::FallingSnowSimulation(	Configuration *conf,
												SceneDescriptor *scene,
												WindField *windField 
	) : conf(conf), scene(scene), windField(windField)
{
	snowflakes = new Snowflake[conf->numSnowflakes];
	nSnowflakes = conf->numSnowflakes;

	for (int i = 0; i < nSnowflakes; i++)
		snowflakes[i].initialize(*conf);
}

FallingSnowSimulation::~FallingSnowSimulation()
{
	delete [] snowflakes;
}

void
FallingSnowSimulation::updateSnowflakes(SnowflakeJob& job, double time)
{
	for (int i = job.start; i < job.end; i++)
	{
		snowflakes[i].takeStep(time, *windField);
		collision(snowflakes[i]);
		//snowflakes[i].advance();
	}
}

void
FallingSnowSimulation::advance(AdvanceSnowflakesJob& job)
{
	for (int i = job.start; i < job.end; i++)
	{
		Snowflake& f = snowflakes[i];

		f.p_prev.x = f.p.x;
		f.p_prev.y = f.p.y;
		f.p_prev.z = f.p.z;
	}
}

void
FallingSnowSimulation::collision(Snowflake& sf)
{
    Box *box;

    // unit vector in snowflake direction
    double ux, uy, uz;
    double len;
    double t;
    // intersection point
    double px, py;
    // matrix value to increase on intersection
    int ix, iy;

        // for all boxes
        for (int ib = 0; ib < conf->nBoxes; ib++)
        {
            box = &(scene->boxes[ib]);

            // check intersection with plane
            if (sf.p.z <= (box->pos.z + box->size.z) &&
                sf.p_prev.z > (box->pos.z + box->size.z) )
            {
                // calculate point of intersection
                ux = sf.p.x - sf.p_prev.x;
                uy = sf.p.y - sf.p_prev.y;
                uz = sf.p.z - sf.p_prev.z;

                if (uz == 0.0f) break;

                len = sqrtf(ux*ux + uy*uy + uz*uz);
                ux /= len;
                uy /= len;
                uz /= len;

                t = (box->pos.z + box->size.z - sf.p_prev.z) / uz;
                px = sf.p_prev.x + t * ux;
                py = sf.p_prev.y + t * uy;

                if ((px >= box->pos.x && px < (box->pos.x + box->size.x)) &&
                    (py >= box->pos.y && py < (box->pos.y + box->size.y)) )
                {
                    px -= box->pos.x;
                    py -= box->pos.y;

                    ix = int((px / box->size.x) * (box->res_x-1) + 0.5);
                    iy = int((py / box->size.y) * (box->res_y-1) + 0.5);

                    box->acc[box->res_x * iy + ix] += 0.3333 * M_PI * sf.diameter*sf.diameter*sf.diameter /
                            ((box->size.x / box->res_x) * (box->size.y/ box->res_y)) *
                            conf->snowflakeVolumeMultiplier;

                    sf.respawn_z(conf->zSize, conf->xSize, conf->ySize);

                } // if intersect inside top of box
            } // if intersect plane
            else // check
            {
                if (sf.p.x < 0.0)
                    sf.respawn_x(conf->xSize, conf->ySize, conf->zSize);
                else if (sf.p.x > conf->xSize)
                    sf.respawn_x(0.0, conf->ySize, conf->zSize);
                else if (sf.p.y < 0.0)
                    sf.respawn_y(conf->ySize, conf->xSize, conf->zSize);
                else if (sf.p.y > conf->ySize)
                    sf.respawn_y(0.0, conf->xSize, conf->zSize);
                else if (sf.p_prev.z < 0.0)
                    sf.respawn_z(conf->zSize, conf->xSize, conf->ySize);
                else if (sf.p.z > conf->zSize)
                    sf.respawn_z(conf->zSize, conf->xSize, conf->ySize);
            }
        } // for boxes

}


