#include "snow_application.h"

SnowApplication::SnowApplication(int argc, char **argv)
{
	int t = 0;
	if (argc == 2 && strcmp(argv[1], "-in") == 0)
		conf.read(std::cin);
	else if (argc == 3 && strcmp(argv[1], "-f") == 0) {
		ifstream infile(argv[2]);
		conf.read(infile);
		infile.close();
	} else {
		ifstream infile("default.scn");
		conf.read(infile);
		infile.close();
	}
	if (argc == 2 && (t = atoi(argv[1])) > 0)
	{
		conf.threads = t;
	}
}

SnowApplication::~SnowApplication()
{
	
}

int 
SnowApplication::run()
{
	Timer timer;
	if (initSDL() || initOpenGL()) 
	{
		SDL_Quit();
		return 1;	
	}

	srand(0);

	conf.printConfiguration();
	SnowSimulator engine(&conf);

	std::cout << std::endl << "Main thread running..." << std::endl;
	
	timer.increment();

	engine.runSimulation();

	timer.increment();

	std::cout << std::endl;
	if (engine.frames > 0)
	{
		std::cout << "Average framerate: " << engine.frames / timer.ddelta() << " fps" << std::endl;
		std::cout << "Running time:      " << timer.ddelta() << " seconds" << std::endl;
	}
	return 0;
}

int 
SnowApplication::initSDL()
{
	if ( SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE | SDL_HWSURFACE) < 0 ) 
	{
    	std::cerr << "Unable to initialize SDL: " << SDL_GetError() << std::endl;
		return 1;
  	}
  	
	atexit(SDL_Quit);
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
  	if ( (conf.state.framebuffer = SDL_SetVideoMode(conf.screenWidth, conf.screenHeight, 0, SDL_OPENGL)) == NULL ) 
  	{
    	std::cerr << "Unable to create OpenGL screen: " << SDL_GetError() << std::endl;
    	return 1;
  	}
  	
  	SDL_WM_SetCaption("Snowstorm", NULL);
  	
  	return 0;
}

int 
SnowApplication::initOpenGL()
{
	glViewport(0, 0, conf.screenWidth, conf.screenHeight);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClearDepth(1.0);
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_COLOR_MATERIAL);
	//glEnable(GL_CULL_FACE);
	glShadeModel (GL_SMOOTH);
	glEnable(GL_TEXTURE_2D);
	
  	return 0;
}


