#include "snow_renderer.h"

SnowRenderer::SnowRenderer(	Configuration *conf,
							SceneDescriptor *scene,
							WindField *windField,
							FallingSnowSimulation *fallingSnow
) 
	: conf(conf), scene(scene), windField(windField), fallingSnow(fallingSnow)
{
	camera = new Camera();
	input = new InputController(conf, camera);

	nTextures = 1;
	textures = new Texture*[nTextures];
	glTextures = new GLuint[nTextures];
	glGenTextures(nTextures, glTextures);
	for (int i = 0; i < nTextures; i++)
	{
		textures[i] = new Texture("images/bricks.jpg");
		glBindTexture(GL_TEXTURE_2D, glTextures[i]);
		textures[i]->bind();
	}

	//fpsDraw = new FramerateDrawer(conf->state.framebuffer, camera, glTextures[nTextures]);

}

SnowRenderer::~SnowRenderer()
{
	//delete fpsDraw;
	delete input;
	delete camera;
	delete textures[0];
	delete [] textures;
	delete [] glTextures;

}

void
SnowRenderer::render()
{
	camera->updatePosition();
	camera->setPerspective();
	glMatrixMode(GL_MODELVIEW);
	glColor3f(0.0, 0.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	setFog();

	if (conf->state.drawPressureField)
		drawPressureField();

	drawSnow();
	drawObjects();
	drawSnowCover();
	if (conf->state.drawWindField)
		drawWindField();

	if (conf->state.drawWindFieldOutline)
		drawWindFieldOutline();

	//fpsDraw->draw();
	SDL_GL_SwapBuffers();
}

void
SnowRenderer::setFog()
{
	GLfloat fogColor[4] = {0.5f, 0.5f, 0.5f, 1.0f};
	glFogi(GL_FOG_MODE, GL_LINEAR);
	glFogfv(GL_FOG_COLOR, fogColor);
	glFogf(GL_FOG_DENSITY, 0.35f);
	glHint(GL_FOG_HINT, GL_DONT_CARE);
	glFogf(GL_FOG_START, 15.0f);
	glFogf(GL_FOG_END, 50.0f);

	if (conf->state.fogEnabled)
	{
		glClearColor(fogColor[0], fogColor[1], fogColor[2], fogColor[3]);
		glEnable(GL_FOG);
	} else 
	{
		glClearColor(0.0, 0.0, 0.0, 1.0);
		glDisable(GL_FOG);
	}
}

void
SnowRenderer::drawWindField()
{
	int i, j, k;
	double dx, dy, dz, x, y, z;

	dx = conf->wfSize.x / conf->Nx;
	dy = conf->wfSize.y / conf->Ny;
	dz = conf->wfSize.z / conf->Nz;
	int Nx = conf->Nx;
	int Ny = conf->Ny;
	
	glMatrixMode(GL_MODELVIEW);
	
	for (k = 0; k < conf->Nz; k++) {
		z = (k + 0.5 * conf->wfSize.z / conf->Nz) * dz + conf->wfPos.z;
		for (j = 0; j < conf->Ny; j++) {
			y = (j + 0.5 * conf->wfSize.y / conf->Ny) * dy + conf->wfPos.y;
			for (i = 0; i < conf->Nx; i++)
			{
				x = (i + 0.5 * conf->wfSize.x / conf->Nx) * dx + conf->wfPos.x;
				glColor3f(0.2, 0.2, 0.7);
				glBegin(GL_LINES);
				glVertex3f(x, y, z);
				glVertex3f(x + windField->m_x[I(i+1,j+1,k+1)], y + windField->m_y[I(i+1,j+1,k+1)], z + windField->m_z[I(i+1,j+1,k+1)]);
				glEnd();
				glColor3f(1.0, 0.0, 0.0);
				glBegin(GL_POINTS);
				glVertex3f(x + windField->m_x[I(i+1,j+1,k+1)], y + windField->m_y[I(i+1,j+1,k+1)], z + windField->m_z[I(i+1,j+1,k+1)]);
				glEnd();
			}
		}
	}
}

void
SnowRenderer::drawPressureField()
{
	int i, j, k;
	double dx, dy, dz, x, y, z;

	dx = conf->xSize / conf->Nx;
	dy = conf->ySize / conf->Ny;
	dz = conf->zSize / conf->Nz;
	int Nx = conf->Nx;
	int Ny = conf->Ny;

	GLUquadricObj* sphere = gluNewQuadric();
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	
	for (k = 0; k < conf->Nz; k++) {
		z = (k + 0.5 * conf->zSize / conf->Nz) * dz;
		for (j = 0; j < conf->Ny; j++) {
			y = (j + 0.5 * conf->ySize / conf->Ny) * dy;
			for (i = 0; i < conf->Nx; i++)
			{
				x = (i + 0.5 * conf->xSize / conf->Nx) * dx;

				glTranslatef(i, j, k);

				if (windField->m_p[I(i+1,j+1,k+1)] >= 0.0f)
				{
					glColor3f(0.2, 0.7, 0.2);
					gluSphere(sphere, windField->m_p[I(i+1,j+1,k+1)] * 10000.0f, 3, 3);
				}
				else 
				{
					glColor3f(0.7, 0.2, 0.2);
					gluSphere(sphere, -1.0f * windField->m_p[I(i+1,j+1,k+1)] * 10000.0f, 3, 3);
				}
				glTranslatef(-i, -j, -k);
			}
		}
	}

	glPopMatrix();
	gluDeleteQuadric(sphere);
}

void
SnowRenderer::drawSnow()
{
	int i;
	Snowflake *snowflakes = fallingSnow->snowflakes;
	glColor3f(1.0, 1.0, 1.0);
	glBegin(GL_QUADS);
	for (i = 0; i < conf->numSnowflakes; i++)
	{
		Snowflake& f = snowflakes[i];
		GLfloat r = f.diameter / 2.8284;  // diving by sqrt(2) to give more reasonable size
		glVertex3f(f.p_prev.x+r, f.p_prev.y, f.p_prev.z+r);
		glVertex3f(f.p_prev.x-r, f.p_prev.y, f.p_prev.z+r);
		glVertex3f(f.p_prev.x-r, f.p_prev.y, f.p_prev.z-r);
		glVertex3f(f.p_prev.x+r, f.p_prev.y, f.p_prev.z-r);

		glVertex3f(f.p_prev.x+r, f.p_prev.y+r, f.p_prev.z);
		glVertex3f(f.p_prev.x-r, f.p_prev.y+r, f.p_prev.z);
		glVertex3f(f.p_prev.x-r, f.p_prev.y-r, f.p_prev.z);
		glVertex3f(f.p_prev.x+r, f.p_prev.y+r, f.p_prev.z);

		glVertex3f(f.p_prev.x, f.p_prev.y+r, f.p_prev.z+r);
		glVertex3f(f.p_prev.x, f.p_prev.y-r, f.p_prev.z+r);
		glVertex3f(f.p_prev.x, f.p_prev.y-r, f.p_prev.z-r);
		glVertex3f(f.p_prev.x, f.p_prev.y+r, f.p_prev.z-r);

	}
	glEnd();
}

void
SnowRenderer::drawSnowCover()
{
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_NORMAL_ARRAY);
	glEnableClientState(GL_COLOR_ARRAY);
	
	glVertexPointer(3, GL_FLOAT, sizeof(struct snow_vertex), &scene->vertArray->x);
	glNormalPointer(GL_FLOAT, sizeof(struct snow_vertex), &scene->vertArray->nx);
	glColorPointer(3, GL_FLOAT, sizeof(struct snow_vertex), &scene->vertArray->r);

	//glEnable(GL_BLEND);
	//glBlendFunc(GL_SRC_ALPHA,GL_ONE);

	glDrawArrays(GL_TRIANGLES, 0, scene->vertices);
	//glDisable(GL_BLEND);

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);
	glDisableClientState(GL_COLOR_ARRAY);

/*	glBegin(GL_TRIANGLES);
	for (int i = 0; i < nEdgeGroups; i++)
	{
		EdgeGroup& e = *edgeGroups[i];
		for (int j = 0; j < e.nTriangles; j++)
		{
			Triangle& t = e.triangles[j];
			glColor3f(t.col[0], t.col[1], t.col[2]);

			glVertex3f(t.p1.x, t.p1.y, t.p1.z + t.v1->zLevel);
			glVertex3f(t.p2.x, t.p2.y, t.p2.z + t.v2->zLevel);
			glVertex3f(t.p3.x, t.p3.y, t.p3.z + t.v3->zLevel);

			//glVertex3f(t.p1.x, t.p1.y, t.p1.z);
			//glVertex3f(t.p2.x, t.p2.y, t.p2.z);
			//glVertex3f(t.p3.x, t.p3.y, t.p3.z);

		}
	}
	glEnd();
*/
}

void
SnowRenderer::drawObjects()
{

	glEnable(GL_TEXTURE_2D);
	glBegin(GL_QUADS);
	Vector3D p1, p2, p3;

	for (int i = 0; i < scene->nBoxes; i++)
	{
		Box &b = scene->boxes[i];
		glBindTexture(GL_TEXTURE_2D, glTextures[b.texture]);

		glColor3f(b.color.x, b.color.y, b.color.z);

		// front
		p1 = b.pos;
		p2 = b.pos;
		p2.x = b.pos.x + b.size.x;
		p3 = b.pos;
		p3.z = b.pos.z + b.size.z;
		drawTiledSurface(b.pos, p2, p3, 0.3);
		
		// back
		p1 = b.pos;
		p1.x += b.size.x;
		p1.y += b.size.y;
		p2 = b.pos;
		p2.y += b.size.y;
		p3 = b.pos + b.size;
		drawTiledSurface(p1, p2, p3, 0.3);

		// left
		p1 = b.pos;
		p1.y += b.size.y;
		p2 = b.pos;
		p3 = b.pos;
		p3.y += b.size.y;
		p3.z += b.size.z;
		drawTiledSurface(p1, p2, p3, 0.3);

		// right
		p1 = b.pos;
		p1.x += b.size.x;
		p2 = b.pos;
		p2.x += b.size.x;
		p2.y += b.size.y;
		p3 = b.pos;
		p3.x += b.size.x;
		p3.z += b.size.z;
		drawTiledSurface(p1, p2, p3, 0.3);

	}
	glEnd();
	glDisable(GL_TEXTURE_2D);
}

void
SnowRenderer::drawTiledSurface(Vector3D p1, Vector3D p2, Vector3D p3, float textureSize)
{

	float x = sqrtf((p2.x-p1.x)*(p2.x-p1.x)+ (p2.y-p1.y)*(p2.y-p1.y)+(p2.z-p1.z)*(p2.z-p1.z));
	float y = sqrtf((p3.x-p1.x)*(p3.x-p1.x)+ (p3.y-p1.y)*(p3.y-p1.y)+(p3.z-p1.z)*(p3.z-p1.z));
	int nx = int(x / textureSize);
	int ny = int(y / textureSize);
	Vector3D xIncr = (p2 - p1).unit() * textureSize;
	Vector3D yIncr = (p3 - p1).unit() * textureSize;
	Vector3D base;

	for (int i = 0; i < nx; i++)
		for (int j = 0; j < ny; j++)
		{
			base.x = p1.x + xIncr.x * i + yIncr.x * j;
			base.y = p1.y + xIncr.y * i + yIncr.y * j;
			base.z = p1.z + xIncr.z * i + yIncr.z * j;

			glTexCoord2f( 0.0f, 1.0f );
			glVertex3f(base.x, base.y, base.z);

			glTexCoord2f( 1.0f, 1.0f );
			glVertex3f(base.x + xIncr.x, base.y + xIncr.y, base.z + xIncr.z);

			glTexCoord2f( 1.0f, 0.0f );
			glVertex3f(base.x + xIncr.x + yIncr.x, base.y + xIncr.y + yIncr.y, base.z + xIncr.z + yIncr.z);

			glTexCoord2f( 0.0f, 0.0f );
			glVertex3f(base.x + yIncr.x, base.y + yIncr.y, base.z + yIncr.z);
		}

	float resty = (y - float(ny)*textureSize) / textureSize;

	for (int i = 0; i < nx; i++)
	{
		base.x = p1.x + xIncr.x * i + yIncr.x * ny;
		base.y = p1.y + xIncr.y * i + yIncr.y * ny;
		base.z = p1.z + xIncr.z * i + yIncr.z * ny;

		glTexCoord2f( 0.0f, 1.0f);
		glVertex3f(base.x, base.y, base.z);

		glTexCoord2f( 1.0f, 1.0f);
		glVertex3f(base.x + xIncr.x, base.y + xIncr.y, base.z + xIncr.z);

		glTexCoord2f( 1.0f, 1-resty);
		glVertex3f(base.x + xIncr.x + yIncr.x*resty, base.y + xIncr.y + yIncr.y*resty, base.z + xIncr.z + yIncr.z*resty);

		glTexCoord2f( 0.0f, 1-resty);
		glVertex3f(base.x + yIncr.x*resty, base.y + yIncr.y*resty, base.z + yIncr.z*resty);

	}

	float restx = (x - float(nx)*textureSize) / textureSize;

	for (int j = 0; j < ny; j++)
	{
		base.x = p1.x + xIncr.x * nx + yIncr.x * j;
		base.y = p1.y + xIncr.y * nx + yIncr.y * j;
		base.z = p1.z + xIncr.z * nx + yIncr.z * j;

		glTexCoord2f( 0.0f, 1.0f);
		glVertex3f(base.x, base.y, base.z);

		glTexCoord2f( restx, 1.0f);
		glVertex3f(base.x + xIncr.x*restx, base.y + xIncr.y*restx, base.z + xIncr.z*restx);

		glTexCoord2f( restx, 0.0f);
		glVertex3f(base.x + xIncr.x*restx + yIncr.x, base.y + xIncr.y*restx + yIncr.y*restx, base.z + xIncr.z*restx + yIncr.z);

		glTexCoord2f( 0.0f, 0.0f);
		glVertex3f(base.x + yIncr.x, base.y + yIncr.y, base.z + yIncr.z);

	}
		base.x = p1.x + xIncr.x * nx + yIncr.x * ny;
		base.y = p1.y + xIncr.y * nx + yIncr.y * ny;
		base.z = p1.z + xIncr.z * nx + yIncr.z * ny;

		glTexCoord2f( 0.0f, 1.0f);
		glVertex3f(base.x, base.y, base.z);

		glTexCoord2f( restx, 1.0f);
		glVertex3f(base.x + xIncr.x*restx, base.y + xIncr.y*restx, base.z + xIncr.z*restx);

		glTexCoord2f( restx, 1-resty);
		glVertex3f(base.x + xIncr.x*restx + yIncr.x*resty, base.y + xIncr.y*restx + yIncr.y*resty, base.z + xIncr.z*restx + yIncr.z*resty);

		glTexCoord2f( 0.0f, 1-resty);
		glVertex3f(base.x + yIncr.x*resty, base.y + yIncr.y*resty, base.z + yIncr.z*resty);


}

void 
SnowRenderer::drawWindFieldOutline()
{
	Vector3D p = conf->wfPos;
	p.z += 0.001;
	Vector3D s = conf->wfSize;
	glColor3f(1.0f, 0.0f, 0.0f);
	glBegin(GL_LINES);
	glVertex3f(p.x, p.y, p.z);
	glVertex3f(p.x + s.x, p.y, p.z);
	glEnd();
	glBegin(GL_LINES);
	glVertex3f(p.x, p.y, p.z);
	glVertex3f(p.x, p.y, p.z + s.z);
	glEnd();
	glBegin(GL_LINES);
	glVertex3f(p.x, p.y, p.z + s.z);
	glVertex3f(p.x + s.x, p.y, p.z + s.z);
	glEnd();
	glBegin(GL_LINES);
	glVertex3f(p.x + s.x, p.y, p.z + s.z);
	glVertex3f(p.x + s.x, p.y, p.z);
	glEnd();

	glBegin(GL_LINES);
	glVertex3f(p.x, p.y + s.y, p.z);
	glVertex3f(p.x + s.x, p.y + s.y, p.z);
	glEnd();
	glBegin(GL_LINES);
	glVertex3f(p.x, p.y + s.y, p.z);
	glVertex3f(p.x, p.y + s.y, p.z + s.z);
	glEnd();
	glBegin(GL_LINES);
	glVertex3f(p.x, p.y + s.y, p.z + s.z);
	glVertex3f(p.x + s.x, p.y + s.y, p.z + s.z);
	glEnd();
	glBegin(GL_LINES);
	glVertex3f(p.x + s.x, p.y + s.y, p.z + s.z);
	glVertex3f(p.x + s.x, p.y + s.y, p.z);
	glEnd();

	glBegin(GL_LINES);
	glVertex3f(p.x, p.y, p.z);
	glVertex3f(p.x, p.y + s.y, p.z);
	glEnd();
	glBegin(GL_LINES);
	glVertex3f(p.x, p.y, p.z + s.z);
	glVertex3f(p.x, p.y + s.y, p.z + s.z);
	glEnd();
	glBegin(GL_LINES);
	glVertex3f(p.x + s.x, p.y, p.z);
	glVertex3f(p.x + s.x, p.y + s.y, p.z);
	glEnd();
	glBegin(GL_LINES);
	glVertex3f(p.x + s.x, p.y, p.z + s.z);
	glVertex3f(p.x + s.x, p.y + s.y, p.z + s.z);
	glEnd();
}
		
void 
SnowRenderer::processSDLEvents()
{
	SDL_Event event;
	
	while (SDL_PollEvent(&event)) 
	{
		switch(event.type)
		{
			case SDL_QUIT:
				conf->state.interrupted = true;
				break;
			case SDL_KEYDOWN:
				input->handleKeyDown(event.key.keysym.sym);
				break;
			case SDL_KEYUP:
				input->handleKeyUp(event.key.keysym.sym);
				break;
		}
		
	}	
}

