#ifndef SNOW_RENDERER_H_
#define SNOW_RENDERER_H_

#include "configuration.h"
#include "scene_descriptor.h"
#include "wind_field.h"
#include "falling_snow_simulation.h"
#include "camera.h"
#include "input_controller.h"
#include "texture.h"
#include "framerate_counter.h"

class SnowRenderer
{
	public:
		SnowRenderer(	Configuration *conf,
						SceneDescriptor *scene,
						WindField *windField,
						FallingSnowSimulation *fallingSnow
					);
		~SnowRenderer();
		void render();
		void processSDLEvents();

	private:
		void drawObjects();
		void drawSnow();
		void drawSnowCover();
		void drawWindField();
		void drawPressureField();
		void drawWindFieldOutline();

		void drawTiledSurface(Vector3D p1, Vector3D p2, Vector3D p3, float textureSize);
		void setFog();


		Configuration *conf;
		SceneDescriptor *scene;
		WindField *windField;
		FallingSnowSimulation *fallingSnow;
		InputController *input;
		Camera *camera;

		FramerateCounter count;
		Timer timer;
		unsigned int time;
		unsigned int nextPrintTime;

		GLuint *glTextures;
		Texture **textures;
		int nTextures;
};

#endif
