#ifndef SNOW_SIMULATION_H_
#define SNOW_SIMULATION_H_

#include <iostream>
#include <math.h>

#include "thread.h"
#include "mutex.h"
#include "condition_variable.h"
#include "scene_descriptor.h"
#include "utils.h"
#include "framerate_counter.h"
#include "configuration.h"
#include "vector3D.h"
#include "falling_snow_simulation.h"
#include "snow_renderer.h"
#include "barrier.h"
#include "job.h"

class SimulationThread;

class SnowSimulator
{
	friend class SimulationThread;

	public:
		SnowSimulator(Configuration *conf);
		~SnowSimulator();
		void runSimulation();

		int frames;

	private:
		void initJobs();
		Job *getJob(int threadId, Job *lastJob);
		void finishJob(Job *job);
		void doJob(Job *job);
		void renderJob();
		void incrementTime();
		
		void updateTime();
		void mainLoopSerial();
		void mainLoopPipelined(int threadId);
		void mainLoopDataParallel(int threadId);
		void printFps();

		Configuration *conf;
		SimulationThread **threads;

		SceneDescriptor *scene;
		WindField *windField;
		FallingSnowSimulation *fallingSnow;
		SnowRenderer *renderer;

		Mutex queueMutex;
		Mutex printMutex;
		Barrier *barrier;
		Timer timer;
		FramerateCounter count;
		unsigned long long nextPrintTime;

		int nJobs;
		Job **jobs;
};

class SimulationThread : public Thread
{
	public:
		SimulationThread(int threadId, SnowSimulator *sim);
		void run();

	private:
		int threadId;
		SnowSimulator *sim;
};

#endif /* SNOW_SIMULATION_H_ */
