#include "snowcover_job.h"

#include "utils.h"

SnowcoverJob::SnowcoverJob(Configuration& conf, int threadNo)
{
	yNormalsFrom = new int[conf.nBoxes];
	yNormalsTo = new int[conf.nBoxes];
	xNormalsFrom = new int[conf.nBoxes];
	xNormalsTo = new int[conf.nBoxes];
	yVerticesFrom = new int[conf.nBoxes];
	yVerticesTo = new int[conf.nBoxes];
	xVerticesFrom = new int[conf.nBoxes];
	xVerticesTo = new int[conf.nBoxes];
	yTo = new int[conf.nBoxes];
	yFrom = new int[conf.nBoxes];

	for (int i = 0; i < conf.nBoxes; i++)
	{
		// inside domain for normals
		divideAmongThreads(threadNo, conf.threads, conf.boxes[i].res_y+1, yNormalsFrom[i], yNormalsTo[i]);
		if (threadNo == 0)
			yNormalsFrom[i] += 2;
		if (threadNo == conf.threads-1)
			yNormalsTo[i] -= 1;

		divideAmongThreads(threadNo, conf.threads, conf.boxes[i].res_x+1, xNormalsFrom[i], xNormalsTo[i]);
		if (threadNo == 0)
			xNormalsFrom[i] += 2;
		if (threadNo == conf.threads-1)
			xNormalsTo[i] -= 1;

		// inside domain for positions
		divideAmongThreads(threadNo, conf.threads, conf.boxes[i].res_y+1, yVerticesFrom[i], yVerticesTo[i]);
		if (threadNo == 0)
			yVerticesFrom[i] += 1;
		if (threadNo == conf.threads-1)
			yVerticesTo[i] -= 1;

		divideAmongThreads(threadNo, conf.threads, conf.boxes[i].res_x+1, xVerticesFrom[i], xVerticesTo[i]);

		if (threadNo == 0)
			xVerticesFrom[i] += 1;
		if (threadNo == conf.threads-1)
			xVerticesTo[i] -= 1;

		divideAmongThreads(threadNo, conf.threads, conf.boxes[i].res_y+1, yFrom[i], yTo[i]);
	}
	type = JOB_SNOWCOVER;
	desc = "Snowcover job ";
	desc += '0' + threadNo;
}

void
SnowcoverJob::dump()
{
	cout << desc << ": " << endl;
	for (int i = 0; i < 4; i++)
	{
	cout << "xNormals: " << xNormalsFrom[i] << " - " << xNormalsTo[i] << endl;
	cout << "yNormals: " << yNormalsFrom[i] << " - " << yNormalsTo[i] << endl;
	cout << "xVertices: " << xVerticesFrom[i] << " - " << xVerticesTo[i] << endl;
	cout << "yVertices: " << yVerticesFrom[i] << " - " << yVerticesTo[i] << endl;
	}
	cout << endl;
}

SnowcoverJob::~SnowcoverJob()
{
	delete [] yNormalsFrom;
	delete [] yNormalsTo;
	delete [] xNormalsFrom;
	delete [] xNormalsTo;
	delete [] yVerticesFrom;
	delete [] yVerticesTo;
	delete [] yFrom;
	delete [] yTo;
	delete [] xVerticesFrom;
	delete [] xVerticesTo;
}
