#ifndef SNOWFLAKE_H_
#define SNOWFLAKE_H_

#include <math.h>

#include "vector3D.h"
#include "configuration.h"
#include "wind_field.h"

class WindField;

class Snowflake
{
	public:
		float diameter;              // radius of snowflake
		float rho;                   // density
		float m;
		float Vmaxz;                 // "steady state" maximum falling speed

		float rotRadius;             // radius of rotation
		float rotOmega;              // rotational speed

		Vector3D Fgravity;

		Vector3D p;                   // "working" position
		Vector3D p_prev;              // previous position
		Vector3D V;                   // velocity
		double rotTheta;              // rotational position

		static const double g; 

		void takeStep(double dt, WindField& wf);
		void initialize(Configuration& conf);
		void respawn_x(float x, float y, float z);
		void respawn_y(float x, float y, float z);
		void respawn_z(float x, float y, float z);
		void spawn(float x, float y, float z);
		void advance();
};

class PaperSnowflake : public Snowflake
{
	public:
		PaperSnowflake() {
			_kPerp = 10.0;
			_kPara = 0.1;
			_rho = 0.1;
			_l = 0.1;
		}

		double _u, _v, _omega, _theta; // tumble

		double _kPerp, _kPara, _rho, _l;

		void update(double dt);

		double inline du(double u, double v, double omega, double theta);
		double inline dv(double u, double v, double omega, double theta);
		double inline dOmega(double u, double v, double omega, double theta);
		double inline dTheta(double u, double v, double omega, double theta);

};

#endif /* SNOWFLAKE_H_ */
