#include "vector3D.h"

#include <math.h>

Vector3D::Vector3D()
{
	x = 0;
	y = 0;
	z = 0;
}
Vector3D::Vector3D(float x, float y, float z)
{
	this->x = x;
	this->y = y;
	this->z = z;
}

Vector3D& Vector3D::operator= (Vector3D v)
{
	x = v.x;
	y = v.y;
	z = v.z;
	return *this;
}

Vector3D Vector3D::operator+ (Vector3D v)
{
	return Vector3D(x + v.x, y + v.y, z + v.z);
}

Vector3D Vector3D::operator- (Vector3D v)
{
	return Vector3D(x - v.x, y - v.y, z - v.z);
}

Vector3D Vector3D::operator+ (float val)
{
	return Vector3D(x + val, y + val, z + val);
}


Vector3D Vector3D::operator- (float val)
{
	return Vector3D(x - val, y - val, z - val);
}


Vector3D Vector3D::operator* (float val)
{
	return Vector3D(x * val, y * val, z * val);
}

Vector3D Vector3D::operator/ (float val)
{
	return Vector3D(x / val, y / val, z / val);
}

Vector3D& Vector3D::operator+= (Vector3D v)
{
	x += v.x;
	y += v.y;
	z += v.z;
	return *this;
}

Vector3D& Vector3D::operator+= (float val)
{
	x += val;
	y += val;
	z += val;
	return *this;
}

Vector3D& Vector3D::operator-= (Vector3D v)
{
	x -= v.x;
	y -= v.y;
	z -= v.z;
	return *this;
}

Vector3D& Vector3D::operator-= (float val)
{
	x -= val;
	y -= val;
	z -= val;
	return *this;
}

Vector3D& Vector3D::operator*= (float val)
{
	x *= val;
	y *= val;
	z *= val;
	return *this;
}

Vector3D& Vector3D::operator/= (float val)
{
	x /= val;
	y /= val;
	z /= val;
	return *this;
}

Vector3D Vector3D::operator- ()
{
	return Vector3D(-x, -y, -z);
}

float Vector3D::dot(const Vector3D& v)
{
	return x * v.x + y * v.y + z * v.z;
}

Vector3D Vector3D::cross(const Vector3D& v)
{
	return Vector3D(y * v.z - z * v.x, z * v.x - x * v.z, x * v.y - y * v.x);
}

float Vector3D::length()
{
	return sqrtf(x*x + y*y + z*z);
}

Vector3D Vector3D::unit()
{
	float length = this->length();
	if (length == 0.0)
		return *this;

	return Vector3D(x / length, y / length, z / length);
}

void Vector3D::normalize()
{
	float length = this->length();

	if (length == 0.0)
		return;

	x /= length;
	y /= length;
	z /= length;
}

