#ifndef WIND_FIELD_H_
#define WIND_FIELD_H_

#include "vector3D.h"
#include "configuration.h"
#include "scene_descriptor.h"
#include "advect_job.h"
#include "build_solution_vector_job.h"
#include "solve_poisson_job.h"
#include "project_job.h"

#define VOXEL_MASK_SELF		(1)
#define VOXEL_MASK_RIGHT	(2)
#define VOXEL_MASK_LEFT 	(4)
#define VOXEL_MASK_BOTTOM	(8)
#define VOXEL_MASK_TOP		(16)
#define VOXEL_MASK_FRONT	(32)
#define VOXEL_MASK_BACK		(64)

#define I(i,j,k) ((k)*(Nx+2)*(Ny+2)+(j)*(Nx+2)+(i))

class SceneDescriptor;

class WindField
{
	public:
		WindField(Configuration *conf, SceneDescriptor *scene);
		~WindField();
		void initialize();
		void reset();

		// interpolation
		void getVelocity(Vector3D *pos, Vector3D *ret);

		void addForces(double dt);
		void advect(AdvectJob& job, double dt);
		void project(ProjectJob& job, double dt);
		void buildSolutionVector(BuildSolutionVectorJob& job, double dt);
		void solvePoisson(SolvePoissonJob& job);
		void setVelocityBoundaries(float *x, float *y, float *z,
							int zFrom, /* indices for parallelization */
							int zTo,
							int bigZFrom,
							int bigZTo, 
							int bigYFrom, 
							int bigYTo);
		void setBoundaries(float *f, 
							int zFrom, /* indices for parallelization */
							int zTo,
							int bigZFrom,
							int bigZTo, 
							int bigYFrom, 
							int bigYTo);
		void buildObstacles();

		Configuration *conf;
		SceneDescriptor *scene;
		int Nx, Ny, Nz, N;
		float h;
		float xSize, ySize, zSize;

		// solution vector
		float *m_b;
		
		// velocity field
		float *m_x, *m_xPrev, *m_xStar;
		float *m_y, *m_yPrev, *m_yStar;
		float *m_z, *m_zPrev, *m_zStar;

		// pressure field
		float *m_p;

		// bit-mask describing the occupation of obstacles
		int *m_obstacles;
};

#endif
